#SingleInstance Ignore
#UseHook
Menu, Tray, Icon, icon.ico

global Hotkeys := Object()

; ====================================
;             CONFIGURATION
; ====================================

; Set your own (unlimited amount of) hotkeys with the respective FPS values
; List of possible keys: https://www.autohotkey.com/docs/KeyList.htm
; Note: Don't use the asterisk (*) prefix for hotkeys as it bugs out RTSS!
Hotkeys["F2"] := 10
Hotkeys["F3"] := 160
Hotkeys["F4"] := 170

; Set the default FPS to toggle back to
global DefaultFPS := 0

; Make consecutive hotkey presses directly switch to a second FPS value instead of going back to the default one first
global SwitchFPSDirectly := true

; Enable RTSS switching to the "DOOMx64vk.exe" profile upon script startup
global SwitchToDOOMProfile := true

; Restrict hotkey usage to the active DOOM window
global RestrictHotkeysToDOOM := true

; Only set this if you want to override the automatically fetched path of "RTSS.exe"
global RTSSPath = ""

; ====================================
; DON'T TOUCH ANYTHING BELOW THIS LINE
; ====================================

global CurrentFPS = DefaultFPS

RestartScriptAsAdminIfNecessary()
StartRTSSIfNecessary()
SetupHotkeys()

Return

RestartScriptAsAdminIfNecessary()
{
  FullCommandline := DllCall("GetCommandLine", "str")
  if not (A_IsAdmin or RegExMatch(FullCommandline, " /restart(?!\S)"))
  {
    try
    {
      if A_IsCompiled
        Run *RunAs "%A_ScriptFullPath%" /restart
      else
        Run *RunAs "%A_AhkPath%" /restart "%A_ScriptFullPath%"
    }
    
    ExitApp
  }
}

StartRTSSIfNecessary()
{
  if (RTSSPath = "")
  {
    RegRead, RTSSPath, HKEY_LOCAL_MACHINE\SOFTWARE\WOW6432Node\Unwinder\RTSS, InstallPath
    
    if ErrorLevel = 1
      RegRead, RTSSPath, HKEY_CURRENT_USER\SOFTWARE\WOW6432Node\Unwinder\RTSS, InstallPath
  }    

  if !IsProcessRunning("RTSS.exe")
  {
    if FileExist(RTSSPath)
    {
      Run *RunAs "%RTSSPath%"
        
      if SwitchToDOOMProfile
        SelectRTSSProfile()
    }
    else
      MsgBox,, RTSS not found, % "The path '" RTSSPath "' doesn't exist. RTSS couldn't be started. Please make sure RTSS is running to enable FPS toggling."
  }
  else if SwitchToDOOMProfile
    SelectRTSSProfile()
}

SelectRTSSProfile()
{
  WinActivate, ahk_exe RTSS.exe
  WinWaitActive, ahk_exe RTSS.exe

  WinGetPos, RTSSWindowX, RTSSWindowY, RTSSWindowWidth, RTSSWindowHeight, ahk_exe RTSS.exe
  
  CoordMode Pixel  
  ImageSearch, XPos, YPos, RTSSWindowX, RTSSWindowY, (RTSSWindowX + RTSSWindowWidth), (RTSSWindowY + RTSSWindowHeight), *30 *TransBlack rtss_doomx64vk.png

  if ErrorLevel = 0
  {
    CoordMode, Mouse, Screen
    MouseGetPos, MouseXPosBackup, MouseYPosBackup
    
    Click, %XPos% %YPos%
    
    MouseMove, %MouseXPosBackup%, %MouseYPosBackup%, 0
  }
}


SetupHotkeys()
{
  if RestrictHotkeysToDOOM
    Hotkey, IfWinActive, ahk_class Zion

  For Key in Hotkeys
    Hotkey, %Key%, ToggleFPS
}


IsProcessRunning(Executable)
{
  Process, Exist, %Executable%
  
  if !ErrorLevel
    Return false
  else
    Return true
}

ToggleFPS()
{
  FPS := Hotkeys[A_ThisHotkey]
  
  if (SwitchFPSDirectly ? (CurrentFPS == FPS) : (CurrentFPS != DefaultFPS))
    SetFPSTo(DefaultFPS)
  else
    SetFPSTo(FPS)
  
  KeyWait, %A_ThisHotkey%
}

SetFPSTo(FPS)
{
  ControlClick, x480 y230, ahk_exe RTSS.exe,,,
        
  Digits := StrSplit(FPS)
      
  for Index, Digit in Digits
    ControlSend, ahk_parent, {%Digit%}, ahk_exe RTSS.exe,,,
    
  ControlSend, ahk_parent, {Enter}, ahk_exe RTSS.exe,,,
  
  CurrentFPS := FPS  
}